@@ -1,6 +1,6 @@
 # The R Commander and command logger
 
-# last modified 4 August 06 by J. Fox
+# last modified 22 September 06 by J. Fox
 #   slight changes 12 Aug 04 by Ph. Grosjean 
 
 Commander <- function(){
@@ -11,7 +11,7 @@ Commander <- function(){
       warning("The R Commander is already open.")
       return(invisible(NULL))
     }
-    RcmdrVersion <- "1.2-0"
+    RcmdrVersion <- "1.2-1"
     if (is.SciViews()) return(invisible(svCommander(Version=RcmdrVersion))) # +PhG
     setOption <- function(option, default, global=TRUE) {
         opt <- if (is.null(current[[option]])) default else current[[option]]
@@ -149,10 +149,16 @@ Commander <- function(){
     setOption("error.text.color", "red")
     setOption("warning.text.color", "darkgreen")
     setOption("multiple.select.mode", "extended")
-    setOption("suppress.X11.warnings", .Platform$GUI == "X11") # to address problem in Linux
+    setOption("suppress.X11.warnings", 
+        interactive() && .Platform$GUI == "X11" && getRversion() < "2.4.0") # to address problem in Linux
     setOption("showData.threshold", 100)
     setOption("retain.messages", TRUE)
     setOption("crisp.dialogs",  (.Platform$OS.type == "windows") && (getRversion() >= "2.1.1"))
+    if (getRcmdr("suppress.X11.warnings")) {
+        putRcmdr("messages.connection", file(open = "w+"))
+        sink(getRcmdr("messages.connection"), type="message")
+#        putRcmdr("length.messages", 0)
+        }
     if (.Platform$OS.type != "windows") {
         putRcmdr("oldPager", options(pager=RcmdrPager))
         default.font.size <- as.character(setOption("default.font.size", 12, global=FALSE))
@@ -161,11 +167,6 @@ Commander <- function(){
         .Tcl(paste("option add *font ", default.font, sep=""))
         }
     if (getRcmdr("crisp.dialogs")) tclServiceMode(on=FALSE)
-    if (getRcmdr("suppress.X11.warnings")) {
-        putRcmdr("messages.connection", textConnection(".messages", open = "w", local=FALSE))
-        sink(getRcmdr("messages.connection"), type="message")
-        putRcmdr("length.messages", 0)
-        }
     putRcmdr("commanderWindow", tktoplevel())
     .commander <- CommanderWindow()
     placement <- setOption("placement", "-40+20", global=FALSE)
@@ -174,7 +175,6 @@ Commander <- function(){
     tkwm.protocol(.commander, "WM_DELETE_WINDOW", closeCommander)
     topMenu <- tkmenu(.commander)
     tkconfigure(.commander, menu=topMenu)
-#    .commander.done <<- tclVar("0") # to address problem in Debian Linux
     source.files <- list.files(etc, pattern="\\.[Rr]$")
     for (file in source.files) {
         source(file.path(etc, file))
@@ -184,6 +184,8 @@ Commander <- function(){
     .Menus <- menus <- list()
     menuItems <- 0
     for (m in 1:nrow(Menus)){
+        install <- Menus[m, 7]
+        if ((install != "") && (!eval(parse(text=install)))) next  
         if (Menus[m, 1] == "menu") {
             position <- 0
             assign(Menus[m, 2], tkmenu(eval(parse(text=Menus[m, 3])), tearoff=FALSE))
@@ -295,9 +297,6 @@ Commander <- function(){
                 current.line <- sub(")", ", envir=.GlobalEnv)", current.line)
                 justDoIt(current.line)
                 }
-            else if (length(grep("^hist\\(", current.line)) > 0){
-                justDoIt(paste("plot(", current.line, ")", sep=""))
-                }
             else if (any(sapply(exceptions,
                     function(.x) length(grep(paste("^", .x, "\\(", sep=""), current.line)) > 0))){
                 justDoIt(current.line)
@@ -451,7 +450,7 @@ Commander <- function(){
         .commander.done <<- tclVar("0") 
         tkwait.variable(.commander.done)
         }
-    if (!packageAvailable("rgl")) Message(gettextRcmdr("The rgl package is absent; 3D plots are unavailable."), type="warning")
+##    if (!packageAvailable("rgl")) Message(gettextRcmdr("The rgl package is absent; 3D plots are unavailable."), type="warning")
     Message(paste(gettextRcmdr("R Commander Version "), getRcmdr("RcmdrVersion"), ": ", date(), sep=""))
     }
 
@@ -478,20 +477,21 @@ logger <- function(command){
 justDoIt <- function(command) {
     Message()
     if (!getRcmdr("suppress.X11.warnings")){
-        messages.connection<- textConnection(".messages", open="w", local=TRUE)
+        messages.connection <- file(open="w+")
         sink(messages.connection, type="message")
         on.exit({
             sink(type="message")
             close(messages.connection)
             })
         }
+    else messages.connection <- getRcmdr("messages.connection")
     capture.output(result <- try(eval(parse(text=command), envir=.GlobalEnv), silent=TRUE))
     if (class(result)[1] ==  "try-error"){
         Message(message=paste(strsplit(result, ":")[[1]][2]), type="error")
         tkfocus(CommanderWindow())
         return()
         }
-    checkWarnings(.messages)
+    checkWarnings(readLines(messages.connection))
     result
     }
 
@@ -505,14 +505,15 @@ doItAndPrint <- function(command, log=TRUE) {
         eval(parse(text=paste("options(width=", floor(width), ")", sep="")))
         }
     if (!getRcmdr("suppress.X11.warnings")){
-        messages.connection <- textConnection(".messages", open="w", local=TRUE)
+        messages.connection <- file(open="w+")
         sink(messages.connection, type="message")
         on.exit({
             sink(type="message")
             close(messages.connection)
             })
         }
-    output.connection <- textConnection(".Output", open="w", local=TRUE)
+    else messages.connection <- getRcmdr("messages.connection")
+    output.connection <- file(open="w+")
     sink(output.connection, type="output")
     on.exit({
         if (!.console.output) sink(type="output") # if .console.output, output connection already closed
@@ -527,6 +528,7 @@ doItAndPrint <- function(command, log=TRUE) {
         return()
         }
     if (isS4object(result)) show(result) else print(result)
+    .Output <- readLines(output.connection)
     if (.Output[length(.Output)] == "NULL") .Output <- .Output[-length(.Output)] # suppress "NULL" line at end of output
     if (length(.Output) != 0) {  # is there output to print?
         if (.console.output) {
@@ -541,42 +543,35 @@ doItAndPrint <- function(command, log=TRUE) {
         }
     else if (.console.output) sink(type="output")
     # errors already intercepted, display any warnings
-    checkWarnings(.messages)
+    checkWarnings(readLines(messages.connection))
     result
     }
 
 checkWarnings <- function(messages){
-#    if (is.SciViews()) return(invisible()) # PhG: added for SciViews compatibility 
     if (getRcmdr("suppress.X11.warnings")){
-            length.messages <- length(messages)
-            last.length.messages <- getRcmdr("length.messages")
-            if (length.messages > last.length.messages){
-                current.messages <- messages[(last.length.messages + 1):length.messages]
-                putRcmdr("length.messages", length.messages)
-                # suppress X11 warnings (origin at this point unclear)
-                X11.warning <- grep("^Warning\\: X11 protocol error\\: BadWindow \\(invalid Window parameter\\)",
-                    current.messages)
-                if (length(X11.warning) > 0){
-                    current.messages <- current.messages[-X11.warning]
-                    if (length(current.messages) == 0) Message()
-                    }
-                else if (length(current.messages) > 10) {
-                    current.messages <- c(paste(length(current.messages), "warnings."),
-                        gettextRcmdr("First and last 5 warnings:"), 
-                            head(current.messages,5), ". . .", tail(current.messages, 5))
-                    Message(message=paste(current.messages, collapse="\n"), type="warning")
+            X11.warning <- grep("^Warning\\: X11 protocol error\\: BadWindow \\(invalid Window parameter\\)",
+                messages)
+            if (length(X11.warning) > 0){
+                messages <- messages[-X11.warning]
                 }
-            }
-        }
+            if (length(messages) == 0) Message()
+            else if (length(messages) > 10) {
+                messages <- c(paste(length(messages), "warnings."),
+                    gettextRcmdr("First and last 5 warnings:"), 
+                        head(messages,5), ". . .", tail(messages, 5))
+                Message(message=paste(messages, collapse="\n"), type="warning")
+                }
+        else Message(message=paste(messages, collapse="\n"), type="warning")
+        }                        
     else{
-#        if (length(messages) == 0) return()
-        if (length(messages == 0)) Message()
+        if (length(messages) == 0) Message()
         else if (length(messages) > 10){ 
             messages <- c(paste(length(messages), "warnings."),
                 gettextRcmdr("First and last 5 warnings:"), 
                     head(messages, 5), ". . .", tail(messages, 5))
             Message(message=paste(messages, collapse="\n"), type="warning")
             }
+        else Message(message=paste(messages, collapse="\n"), type="warning")
         }
     tkfocus(CommanderWindow())
     }
